﻿using PlumbingRepairContracts.BindingModels;
using PlumbingRepairContracts.ViewModels;
using PlumbingRepairDataModels.Enums;
using PlumbingRepairDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace PlumbingRepairDataBaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }

        public int WorkId { get; private set; }
        public int ClientId {  get; private set; }

        public string WorkName { get; private set; } = string.Empty;

        [Required]
        public int Count { get; private set; }

        [Required]
        public double Sum { get; private set; }

        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

        [Required]
        public DateTime DateCreate { get; private set; }

        public DateTime? DateImplement { get; private set; }

        public virtual Work Work { get; set; }
		public virtual Client Client { get; set; }

		public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }

            return new Order()
            {
                Id = model.Id,
                WorkId = model.WorkId,
                WorkName = model.WorkName,
				ClientId = model.ClientId,
				Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            Status = model.Status;
            DateImplement = model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            WorkId = WorkId,
            WorkName = Work.WorkName,
            ClientId = ClientId,
            ClientFIO = Client.ClientFIO,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
    }
}
